<?php

namespace Drupal\blazy\Dejavu;

/**
 * A Trait common for optional views style plugins for slick, etc.
 *
 * This Trait is not used by blazy, but for blazy-related modules to
 * provide more advanced views style. Implementor must import also
 * Drupal\blazy\BlazyManagerTrait, or override the Blazy manager accordingly.
 */
trait BlazyStylePluginTrait {

  /**
   * Returns available fields for select options.
   */
  public function getDefinedFieldOptions($defined_options = []) {
    $fields = $this->view->display_handler->get_handlers('field');

    $options = [];
    $classes = [
      'list_text',
      'entityreference',
      'taxonomy_term_reference',
      'text',
    ];
    foreach ($fields as $field => $handler) {
      if (isset($handler->field_info)) {
        $type = $handler->field_info['type'];

        switch ($type) {
          case 'file':
          case 'image':
          case 'youtube':
          case 'video_embed_field':
            $options['images'][$field] = $handler->ui_name();
            $options['overlays'][$field] = $handler->ui_name();
            $options['thumbnails'][$field] = $handler->ui_name();
            break;

          case 'list_text':
            $options['layouts'][$field] = $handler->ui_name();
            break;

          case 'entityreference':
          case 'text':
          case 'text_long':
          case 'text_with_summary':
          case 'link_field':
            $options['links'][$field] = $handler->ui_name();
            $options['titles'][$field] = $handler->ui_name();
            if ($type != 'link_field') {
              $options['thumb_captions'][$field] = $handler->ui_name();
            }
            break;
        }
        if (in_array($type, $classes)) {
          $options['classes'][$field] = $handler->ui_name();
        }
      }

      // Content: title is not really a field, unless title.module installed.
      if ($handler->field == 'title') {
        $options['classes'][$field] = $handler->ui_name();
        $options['titles'][$field] = $handler->ui_name();
        $options['thumb_captions'][$field] = $handler->ui_name();
      }

      if ($handler->field == 'nothing') {
        $options['classes'][$field] = $handler->ui_name();
      }

      if (in_array($handler->field, ['nid', 'nothing', 'view_node'])) {
        $options['links'][$field] = $handler->ui_name();
        $options['titles'][$field] = $handler->ui_name();
      }

      // Caption can be anything to get custom works going.
      $options['captions'][$field] = $handler->ui_name();
    }

    $definition['plugin_id'] = $this->plugin_name;
    $definition['settings'] = $this->options;
    $definition['current_view_mode'] = $this->view->current_display;

    // Provides the requested fields based on available $options.
    foreach ($defined_options as $key) {
      $definition[$key] = isset($options[$key]) ? $options[$key] : [];
    }

    $contexts = [
      'handler' => $this->view->display_handler,
      'view' => $this->view,
    ];

    drupal_alter('blazy_views_field_options', $definition, $contexts);
    return $definition;
  }

  /**
   * Returns the modified renderable image_formatter to support lazyload.
   */
  public function getImageRenderable(array &$settings, $row, $index) {
    $image = $this->isImageRenderable($row, $index, $settings['image']);

    /* @var Drupal\image\Plugin\Field\FieldType\ImageItem $item */
    if (empty($image['raw'])) {
      return $image;
    }

    // If the image has #item property, lazyload may work, otherwise skip.
    // This hustle is to lazyload tons of images -- grids, large galleries,
    // gridstack, mason, with multimedia/ lightboxes for free.
    if ($item = $this->getImageItem($image)) {
      // Supports multiple image styles within a single view such as GridStack,
      // else fallbacks to the defined image style if available.
      if (empty($settings['image_style'])) {
        $image_style = isset($image['rendered']['#image_style']) ? $image['rendered']['#image_style'] : '';
        $settings['image_style'] = empty($settings['image_style']) ? $image_style : $settings['image_style'];
      }

      // Converts image formatter for blazy to reduce complexity with CSS
      // background option, and other options, and still lazyload it.
      $theme = isset($image['rendered']['#theme']) ? $image['rendered']['#theme'] : '';
      if (in_array($theme, ['blazy', 'image_formatter'])) {
        if ($theme == 'blazy') {
          // Pass Blazy field formatter settings into Views style plugin.
          // This allows richer contents such as multimedia/ lightbox for free.
          // Yet, ensures the Views style plugin wins over Blazy formatter,
          // such as with GridStack which may have its own breakpoints.
          $item_settings = array_filter($image['rendered']['#build']['settings']);
          $settings = array_merge($item_settings, array_filter($settings));
        }
        elseif ($theme == 'image_formatter') {
          // Deals with "link to content/image" by formatters.
          $settings['content_url'] = isset($image['rendered']['#url']) ? $image['rendered']['#url'] : '';
          // Prevent images from having absurd height when being lazyloaded.
          $settings['ratio'] = 'fluid';
          if (empty($settings['media_switch']) && !empty($settings['content_url'])) {
            $settings['media_switch'] = 'content';
          }
        }

        // Rebuilds the image for the brand new richer Blazy.
        // With the working Views cache, nothing to worry much.
        $build = ['item' => $item, 'settings' => $settings];
        $image['rendered'] = $this->blazyManager()->getBlazy($build);
      }
    }

    return $image;
  }

  /**
   * Checks if we can work with this formatter, otherwise no go if flattened.
   */
  public function isImageRenderable($row, $index, $field_image = '') {
    if (!empty($field_image) && $image = $this->getFieldRenderable($row, $index, $field_image)) {
      if ($this->getImageItem($image)) {
        return $image;
      }

      // Dump Video embed thumbnail/video/colorbox as is.
      if (isset($image['rendered'])) {
        return $image;
      }
    }
    return [];
  }

  /**
   * Get the image item to work with out of this formatter.
   *
   * All this mess is because Views may render/flatten images earlier.
   */
  public function getImageItem($image) {
    $item = [];

    // Image formatter.
    if (isset($image['raw'])) {
      $item = empty($image['rendered']['#item']) ? [] : (object) $image['rendered']['#item'];

      // Blazy formatter.
      if (isset($image['rendered']['#build'])) {
        $item = (object) $image['rendered']['#build']['item'];
      }
    }
    return $item;
  }

  /**
   * Returns the rendered caption fields.
   */
  public function getCaption($index, $settings = []) {
    $items = [];
    $keys = array_keys($this->view->field);

    if (!empty($settings['caption'])) {
      // Exclude non-caption fields so that theme_views_view_fields() kicks in
      // and only render expected caption fields. As long as not-hidden, each
      // caption field should be wrapped with Views markups.
      $excludes = array_diff_assoc(array_combine($keys, $keys), $settings['caption']);
      foreach ($excludes as $field) {
        $this->view->field[$field]->options['exclude'] = TRUE;
      }

      $items['data'] = $this->view->style_plugin->row_plugin->render($this->view->result[$index]);
    }

    $items['link']    = empty($settings['link']) ? [] : $this->getFieldRendered($index, $settings['link']);
    $items['title']   = empty($settings['title']) ? [] : $this->getFieldRendered($index, $settings['title'], TRUE);
    $items['overlay'] = empty($settings['overlay']) ? [] : $this->getFieldRendered($index, $settings['overlay']);

    return $items;
  }

  /**
   * Returns the rendered layout fields.
   */
  public function getLayout(array &$settings, $index) {
    if (strpos($settings['layout'], 'field_') !== FALSE) {
      $settings['layout'] = strip_tags($this->get_field($index, $settings['layout']));
    }
  }

  /**
   * Returns the rendered field, either string or array.
   */
  public function getFieldRendered($index, $field_name = '', $restricted = FALSE) {
    if (!empty($field_name) && $output = $this->get_field($index, $field_name)) {
      return is_array($output) ? $output : ['#markup' => ($restricted ? filter_xss_admin($output) : $output)];
    }
    return [];
  }

  /**
   * Returns the renderable array of field containing rendered and raw data.
   */
  public function getFieldRenderable($row, $index, $field_name = '', $multiple = FALSE) {
    if (empty($field_name)) {
      return FALSE;
    }

    // Be sure to not check "Use field template" under "Style settings" to have
    // renderable array to work with, otherwise flattened string!
    $field = $this->view->field[$field_name]->handler_type . '_' . $field_name;
    return $multiple && isset($row->{$field}) ? $row->{$field} : (isset($row->{$field}[0]) ? $row->{$field}[0] : '');
  }

  /**
   * Returns the string values for the expected Title, ET label, List, Term.
   */
  public function getFieldString($row, $field_name, $index, $clean = TRUE) {
    $result = '';

    if ($value = $this->get_field($index, $field_name)) {
      $value = is_array($value) ? array_filter($value) : $value;

      if (is_string($value)) {
        // Only respects tags with default CSV, just too much to worry about.
        if (strpos($value, ',') !== FALSE) {
          $tags = array_map('trim', explode(",", $value));
          $rendered_tags = [];
          foreach ($tags as $tag) {
            $rendered_tags[] = $clean ? drupal_clean_css_identifier($tag) : $tag;
          }
          $result = implode(' ', $rendered_tags);
        }
        else {
          $result = $clean ? drupal_clean_css_identifier($value) : $value;
        }
      }
      else {
        // @todo recheck if anything else but value worthwhile.
        $value = isset($value[0]) && !empty($value[0]['value']) ? $value[0]['value'] : '';
        if ($value) {
          $result = $clean ? drupal_clean_css_identifier($value) : $value;
        }
      }
    }

    return empty($result) ? '' : trim(strip_tags($result));
  }

}
