<?php

namespace Drupal\blazy\Plugin\Field\FieldFormatter;

use Drupal\blazy\BlazyGrid;
use Drupal\blazy\BlazyManagerInterface;
use Drupal\blazy\Form\BlazyAdminFormatter;

/**
 * Plugin implementation of the `Blazy File` or `Blazy Image` for Blazy only.
 *
 * @see \Drupal\blazy\Plugin\Field\FieldFormatter\BlazyFileFormatter
 * @see \Drupal\blazy\Plugin\Field\FieldFormatter\BlazyImageFormatter
 */
class BlazyFormatterBlazy extends BlazyFormatterBase {

  /**
   * Constructs a BlazyFormatter instance.
   */
  public function __construct($plugin_id, $field, $instance, BlazyManagerInterface $formatter) {
    parent::__construct($plugin_id, $field, $instance);
    $this->formatter = $formatter;
  }

  /**
   * {@inheritdoc}
   */
  public function admin() {
    if (!isset($this->admin)) {
      $this->admin = new BlazyAdminFormatter($this->formatter);
    }
    return $this->admin;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements($items, $entity) {
    $build = [];
    $files = $this->getEntitiesToView($items);

    // Early opt-out if the field is empty.
    if (empty($files)) {
      return $build;
    }

    // Collects specific settings to this formatter.
    $settings              = $this->buildSettings();
    $settings['blazy']     = TRUE;
    $settings['namespace'] = $settings['item_id'] = $settings['lazy'] = 'blazy';
    $settings['_grid']     = !empty($settings['style']) && !empty($settings['grid']);

    // Build the settings.
    $build = ['settings' => $settings];

    // Modifies settings.
    $this->formatter()->buildSettings($build, $files, $entity);

    // Build the elements.
    $this->buildElements($build, $files);

    // Updates settings.
    $settings = $build['settings'];
    unset($build['settings']);

    // Supports Blazy multi-breakpoint images if provided.
    if (isset($build[0]['#build'])) {
      $this->formatter()->isBlazy($settings, $build[0]['#build']);
    }

    // Build grid if provided.
    if (empty($settings['_grid'])) {
      $build['#blazy'] = $settings;
    }
    else {
      $build = BlazyGrid::build($build, $settings);
    }

    $build['#attached'] = $this->formatter()->attach($settings);

    return $build;
  }

  /**
   * Build the Blazy elements for image and file entity/ media.
   */
  public function buildElements(array &$build, $files) {
    $settings = $build['settings'];
    $item_id = $settings['item_id'];

    if (!empty($settings['caption'])) {
      $settings['caption_attributes']['class'][] = $item_id . '__caption';
    }

    foreach ($files as $delta => $item) {
      // Some settings need to be available before arriving at theme_blazy() so
      // that lightboxes and media switcher can get context to work with.
      // The trouble is they are not always available depending on file entity
      // and other supported modules availability which is not always there.
      $settings['delta'] = $delta;
      $settings['type']  = isset($item->type) ? $item->type : 'image';
      $settings['uri']   = isset($item->uri) ? $item->uri : '';
      $box['item']       = $item;
      $box['settings']   = $settings;

      $this->buildElement($box, $item, $delta);

      // Build caption if so configured.
      if (!empty($settings['caption'])) {
        foreach ($settings['caption'] as $caption) {
          if ($caption_content = array_filter($this->getCaption($item, $caption))) {
            $content['content'] = $caption_content;
            $content['tag'] = $caption == 'title' ? 'h2' : 'div';
            $class = $caption == 'alt' ? 'description' : $caption;
            $content['attributes']['class'][] = $item_id . '__' . str_replace('_', '-', $class);

            $box['captions'][$caption] = $content;
          }
        }
      }

      // Image with grid, responsive image, lazyLoad, and lightbox supports.
      $build[$delta] = $this->formatter()->getBlazy($box);
      unset($box);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildElement(array &$element, $entity, $delta = 0) {
    // Do nothing.
  }

  /**
   * Returns the captions.
   */
  protected function getCaption($item, $caption) {
    return [];
  }

}
