<?php

namespace Drupal\blazy\Plugin\Filter;

use Drupal\blazy\BlazyDefault;
use Drupal\blazy\BlazyManagerInterface;
use Drupal\blazy\Plugin\Field\FieldFormatter\BlazyVideoTrait;

/**
 * Provides a filter to lazyload image, or iframe elements.
 */
class BlazyFilter {

  use BlazyVideoTrait;

  /**
   * The processed text.
   *
   * @var string
   */
  protected $processedText;

  /**
   * The blazy manager service.
   *
   * @var \Drupal\blazy\BlazyManagerInterface
   */
  protected $manager;

  /**
   * Constructs the BlazyFilter object.
   */
  public function __construct(BlazyManagerInterface $manager) {
    $this->manager = $manager;
  }

  /**
   * Return blazy filter info.
   */
  public function filterInfo() {
    $filters['blazy_filter'] = [
      'title'             => t('Lazyload inline images, or video iframes, using Blazy'),
      'cache'             => TRUE,
      'process callback'  => '_blazy_filter_process',
      'settings callback' => '_blazy_filter_settings_form',
      'tips callback'     => '_blazy_filter_tips',
      'weight'            => 3,
      'default settings'  => [
        'filter_tags'  => ['img' => 'img', 'iframe' => 'iframe'],
        'column'       => TRUE,
        'grid'         => TRUE,
        'media_switch' => '',
      ],
    ];

    return $filters;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $filter, $langcode) {
    if (!isset($this->processedText)) {
      $allowed_tags = array_values((array) $filter->settings['filter_tags']);
      if (empty($allowed_tags)) {
        return $text;
      }

      $dom = filter_dom_load($text);
      $settings['grid'] = stristr($text, 'data-grid') !== FALSE;
      $settings['column'] = stristr($text, 'data-column') !== FALSE;
      $settings['media_switch'] = $switch = $filter->settings['media_switch'];
      $settings['lightbox'] = ($switch && in_array($switch, $this->manager->getLightboxes())) ? $switch : FALSE;
      $settings['id'] = 'blazy-filter-' . drupal_random_key(8);
      $settings['plugin_id'] = 'blazy_filter';

      foreach ($allowed_tags as $allowed_tag) {
        $nodes = $dom->getElementsByTagName($allowed_tag);
        if ($nodes->length > 0) {
          $settings['count'] = $nodes->length;
          foreach ($nodes as $delta => $node) {
            if ($node->hasAttribute('data-unblazy')) {
              continue;
            }

            // Build Blazy elements with lazyloaded image, or iframe.
            $settings['delta'] = $delta;
            $settings = array_merge($settings, (array) $this->buildSettings($node, $filter));
            $item = $this->buildImageItem($node, $settings);
            $build = [
              'item' => $item,
              'settings' => $settings,
            ];

            $output = $this->manager->getBlazy($build);
            $altered_html = drupal_render($output);

            // Load the altered HTML into a new DOMDocument, retrieve element.
            $updated_nodes = filter_dom_load($altered_html)->getElementsByTagName('body')
              ->item(0)
              ->childNodes;

            foreach ($updated_nodes as $updated_node) {
              // Import the updated from the new DOMDocument into the original
              // one, importing also the child nodes of the updated node.
              $updated_node = $dom->importNode($updated_node, TRUE);
              $node->parentNode->insertBefore($updated_node, $node);
            }

            // Finally, remove the original blazy node.
            $node->parentNode->removeChild($node);
          }
        }
      }

      if ($settings['column'] || $settings['grid']) {
        $this->buildGrid($dom, $settings);
      }

      $text = filter_dom_serialize($dom);
      $this->processedText = trim($text);
    }

    return $this->processedText;
  }

  /**
   * {@inheritdoc}
   */
  public function tips($filter, $long = FALSE) {
    if ($long) {
      $tips = t('<p><strong>Blazy</strong>: Image or iframe is lazyloaded. To disable, add attribute <code>data-unblazy</code>:</p>
      <ul>
          <li><code>&lt;img data-unblazy /&gt;</code></li>
          <li><code>&lt;iframe data-unblazy /&gt;</code></li>
      </ul>');

      if ($filter->settings['grid'] || $filter->settings['column']) {
        if ($filter->settings['grid']) {
          $tips .= t('<p>To build a grid of images/ videos, add attribute <code>data-grid</code> (only to the first item):
          <ul>
              <li>For images: <code>&lt;img data-grid="1 3 4" /&gt;</code></li>
              <li>For videos: <code>&lt;iframe data-grid="1 3 4" /&gt;</code></li>
              <li>If both media types are present, choose only the first item.</li>
          </ul>');
        }
        if ($filter->settings['column']) {
          $tips .= t('<p>To build a CSS3 Masonry columns of images/ videos, add attribute <code>data-column</code> (only to the first item):
          <ul>
              <li>For images: <code>&lt;img data-column="1 3 4" /&gt;</code></li>
              <li>For videos: <code>&lt;iframe data-column="1 3 4" /&gt;</code></li>
              <li>If both media types are present, choose only the first item.</li>
          </ul>');
        }

        $tips .= t('The numbers represent the amount of grids/ columns for small, medium and large devices respectively, space delimited. Be aware! All media items will be grouped regardless of their placements, unless those disabled via <code>data-unblazy</code>. This is also required if using <b>Image to lightbox</b> (Colorbox, Photobox, PhotoSwipe). Only one block of grids or columns can exist at a time in a particular body text.</p>');
      }

      return $tips;
    }
    else {
      return t('To disable lazyload, add attribute <code>data-unblazy</code> to <code>&lt;img&gt;</code> or <code>&lt;iframe&gt;</code> elements. Examples: <code>&lt;img data-unblazy</code> or <code>&lt;iframe data-unblazy</code>.');
    }
  }

  /**
   * Build the grid.
   *
   * @param \DOMDocument $dom
   *   The HTML DOM object being modified.
   * @param array $settings
   *   The settings array.
   *
   * @todo use BlazyGrid if you can.
   */
  private function buildGrid(\DOMDocument &$dom, array $settings) {
    $xpath = new \DOMXPath($dom);
    $query = $settings['column'] ? 'column' : 'grid';
    $nodes = $xpath->evaluate('//*[contains(@class, "grid")]');
    $grid = FALSE;

    // This is weird, variables not working for xpath?
    if ($query == 'column') {
      $node = $xpath->query('//*[@data-column]');
      if ($node->length > 0 && $node->item(0) && $node->item(0)->hasAttribute('data-column')) {
        $grid = $node->item(0)->getAttribute('data-column');
      }
    }
    else {
      $node = $xpath->query('//*[@data-grid]');
      if ($node->length > 0 && $node->item(0) && $node->item(0)->hasAttribute('data-grid')) {
        $grid = $node->item(0)->getAttribute('data-grid');
      }
    }

    $classes = [];
    if (($first = $nodes[0]) && $grid) {
      // Create the parent grid container, and put it before the first.
      $container = $first->parentNode->insertBefore($dom->createElement('div'), $first);
      $grids = array_map('trim', explode(' ', $grid));
      $classes[] = 'blazy blazy--filter blazy--' . $query . ' block-' . $query;

      foreach (['small', 'medium', 'large'] as $key => $item) {
        if (isset($grids[$key])) {
          $classes[] = $item . '-block-' . $query . '-' . $grids[$key];
        }
      }

      // Add the container classes, and merge grid items into container.
      $container->setAttribute('class', implode(' ', $classes));
      $container->setAttribute('data-blazy', '');

      if (!empty($settings['media_switch'])) {
        $switch = str_replace('_', '-', $settings['media_switch']);
        $container->setAttribute('data-' . $switch . '-gallery', TRUE);
      }

      foreach ($nodes as $node) {
        $container->appendChild($node);
      }
    }
  }

  /**
   * Returns the faked image item for the image, uploaded or hard-coded.
   *
   * @param object $node
   *   The HTML DOM object.
   * @param array $settings
   *   The settings array being modified.
   *
   * @return object
   *   The faked image item.
   */
  private function buildImageItem(&$node, array &$settings = []) {
    $item = new \stdClass();
    $item->uri = $settings['uri'];

    // Responsive image with aspect ratio requires an extra container to work
    // with Align/ Caption images filters.
    $settings['media_attributes']['class'][] = 'media-wrapper media-wrapper--blazy';
    // Copy all attributes of the original node to the $item _attributes.
    if ($node->attributes->length) {
      foreach ($node->attributes as $attribute) {
        // Move classes (align-BLAH,etc) to Blazy container, not image so to
        // work with alignments and aspect ratio.
        if ($attribute->nodeName == 'class') {
          $settings['media_attributes']['class'][] = $attribute->nodeValue;
        }
        else {
          $item->_attributes[$attribute->nodeName] = $attribute->nodeValue;
        }
      }

      if ($settings['column'] || $settings['grid']) {
        if ($node->parentNode->tagName === 'figure') {
          $classes = $node->parentNode->getAttribute('class');
          $classes = (strlen($classes) > 0) ? explode(' ', $classes) : [];
          $classes[] = 'grid';
          $node->parentNode->setAttribute('class', implode(' ', array_unique($classes)));
        }
        else {
          $settings['media_attributes']['class'][] = 'grid';
        }
      }

      $settings['media_attributes']['class'] = array_unique($settings['media_attributes']['class']);
    }

    return $item;
  }

  /**
   * Returns the settings for the current $node.
   *
   * @param object $node
   *   The HTML DOM object.
   * @param object $filter
   *   The filter object.
   *
   * @return array
   *   The settings for the current $node.
   */
  private function buildSettings($node, $filter) {
    $src = $node->getAttribute('src');
    $width = $node->getAttribute('width');
    $height = $node->getAttribute('height');

    if ($src && $node->tagName == 'img') {
      $abs_url = strpos($src, 'http') === FALSE ? DRUPAL_ROOT . $src : $src;
      if (!$width && $data = @getimagesize($abs_url)) {
        list($width, $height) = $data;
      }
    }

    $settings = [
      'ratio' => !$width ? '' : 'fluid',
      'image_url' => $src,
      'media_switch' => $filter->settings['media_switch'],
    ];

    // @todo file_build_uri() makes public://sites/default/files/media/Screen...
    $uri = strpos($src, 'http') === FALSE ? $src : $src;
    if ($node->tagName == 'iframe') {
      $settings['input_url'] = $src;

      $image_url = $settings['image_url'] = $this->getVideoThumbnail($src);
      $uri = !empty($image_url) && strpos($image_url, 'http') === FALSE ? file_build_uri($image_url) : $src;
      $settings['scheme'] = $this->getHost($src);
      $settings['embed_url'] = $this->getVideoEmbedUrl($src);
      $settings['autoplay_url'] = $this->getAutoplayUrl($src);
      $settings['ratio'] = !$width ? '16:9' : 'fluid';
      $settings['type'] = 'video';
    }

    return [
      'blazy' => TRUE,
      'lazy' => 'blazy',
      'uri' => $uri,
      'width' => $width,
      'height' => $height,
    ] + $settings;
  }

  /**
   * Implements callback_filter_settings().
   */
  public function settingsForm($form, &$form_state, $filter) {
    $lightboxes = $this->manager->getLightboxes();

    $elements['filter_tags'] = [
      '#type' => 'checkboxes',
      '#title' => t('Enable HTML tags'),
      '#options' => [
        'img' => t('Image'),
        'iframe' => t('Video iframe'),
      ],
      '#default_value' => empty($filter->settings['filter_tags']) ? [] : array_values((array) $filter->settings['filter_tags']),
      '#description' => t('To disable per item, add attribute <code>data-unblazy</code>.'),
    ];

    $elements['grid'] = [
      '#type' => 'checkbox',
      '#title' => t('Grid Foundation'),
      '#default_value' => $filter->settings['grid'],
    ];

    $elements['column'] = [
      '#type' => 'checkbox',
      '#title' => t('CSS3 Masonry columns'),
      '#default_value' => $filter->settings['column'],
      '#description' => t('Check to support inline grids, or columns. Load both to support any, yet only one can exist at a time in a particular body text.'),
    ];

    $elements['media_switch'] = [
      '#type' => 'select',
      '#title' => t('Media switcher'),
      '#options' => [
        'media' => t('Image to iframe'),
      ],
      '#empty_option' => t('- None -'),
      '#default_value' => $filter->settings['media_switch'],
      '#description' => t('<ul><li><b>Image to iframe</b> will hide iframe behind image till toggled.</li><li><b>Image to lightbox</b> (Colorbox, Photobox, PhotoSwipe, Intense, etc.) requires a grid. Add <code>data-column="1 3 4"</code> or <code>data-grid="1 3 4"</code> to the first image/ iframe only.</li></ul>'),
      '#suffix' => t('Check out <a href="@url1">filter tips</a> for details. Be sure to configure Blazy pages <a href="@url2">here</a>.', [
        '@url1' => url('filter/tips', ['fragment' => 'filter-blazy_filter']),
        '@url2' => url('admin/config/media/blazy', ['fragment' => 'edit-visibility']),
      ]),
    ];

    if (!empty($lightboxes)) {
      foreach ($lightboxes as $lightbox) {
        $elements['media_switch']['#options'][$lightbox] = t('Image to @lightbox', ['@lightbox' => $lightbox]);
      }
    }

    return $elements;
  }

  /**
   * Submit handler for hook_filter_admin_format_form_alter().
   *
   * The hustle is due to non-bubbleable cache metadata at D7, needs to catch
   * at least the enabled settings without querying database.
   */
  public function submitForm($form, &$form_state) {
    $defaults = BlazyDefault::formSettings()['filters'];
    if (isset($form_state['values']['filters']['blazy_filter'])) {
      $blazy = $form_state['values']['filters']['blazy_filter'];
      if ($blazy['status'] == 1) {
        $format = $form_state['values']['format'];
        $settings = &$blazy['settings'];
        $components['filters'] = [];
        foreach ($defaults as $key => $value) {
          if (isset($settings[$key])) {
            $type = gettype($value);
            settype($settings[$key], $type);

            $components['filters'][$format][$key] = $settings[$key];
          }
        }

        // Merge individual flat variables into a single blazy.settings.
        variable_set('blazy.settings', array_merge((array) $this->manager->config(), $components));
      }
    }
  }

}
